﻿using System.ComponentModel.DataAnnotations;
using System.Runtime.Serialization;
using DataAnnotationsExtensions;
using CaspitWebTiles.Api;


namespace CaspitWebTiles.Api.Models
{
    /// <summary>
    /// לקוח או ספק בכספית
    /// </summary>
    [DataContract(Namespace = "http://www.caspit.biz/2013/03")]
    public class CaspitContact : CaspitBase
    {
        /// <summary>
        /// המפתח של הלקוח. 
        /// לקוחות שנוצרים בכספית מקבלים מפתח באופן אוטומטי
        /// המפתח שכספית מייצרת הוא מסוג GUID
        /// לקוחות שנוצרים דרך ה-API מקבלים את המפתח שנשלח לכספית
        /// השדה משמש לזיהוי הלקוח בעת שליפה מכספית
        /// </summary>
        [Key, Required, DataMember, StringLength(50)]
        public string ContactId { get; set; }

        /// <summary>
        /// ערכים אפשריים:
        /// Null or 0 = לקוח
        /// ספק = 1,
        /// לקוח וגם ספק = 2
        /// </summary>
        [DataMember, Range(0,2)]
        public int? ContactType { get; set; }

        /// <summary>
        /// מספר הלקוח בכספית. 
        /// כספית ממספרת את הלקוחות באופן אוטומטי בעת יתירתם
        /// אל תשתמש בו כמפתח לשליפה מכספית, אלא השתמש במפתח.
        /// </summary>
        [DataMember]
        public int? Number { get; set; }
        /// <summary>
        /// שם העסק
        /// </summary>
        [DataMember, StringLength(200)]
        public string BusinessName { get; set; }
        /// <summary>
        /// מספר עוסק מורשה של הלקוח
        /// </summary>
        [DataMember, StringLength(20)]
        public string OsekMorshe { get; set; }
        /// <summary>
        /// שם איש הקשר
        /// </summary>
        [DataMember, StringLength(100)]
        public string Name { get; set; }

        /// <summary>
        /// פרטי הכתובת של הלקוח: רחוב ומספר בית
        /// </summary>
        [DataMember, StringLength(100)]
        public string Address1 { get; set; }

        /// <summary>
        /// פרטי הכתובת של הלקוח: מספר דירה, ת.ד או שכונה.
        /// </summary>
        [DataMember, StringLength(100)]
        public string Address2 { get; set; }
        /// <summary>
        /// שם הישוב
        /// </summary>
        [DataMember, StringLength(100)]
        public string City { get; set; }
        /// <summary>
        /// מיקוד
        /// </summary>
        [DataMember, StringLength(20)]
        public string PostalCode { get; set; }
        /// <summary>
        /// מדינה
        /// </summary>
        [DataMember, StringLength(20)]
        public virtual string Country { get; set; }

        /// <summary>
        /// פרטי הקשר של הלקוח: טלפון
        /// </summary>
        [DataMember, StringLength(20)]
        public string Phone { get; set; }
        /// <summary>
        /// מספר נייד
        /// </summary>
        [DataMember, StringLength(20)]
        public string Mobile { get; set; }
        /// <summary>
        /// מספר פקס
        /// </summary>
        [DataMember, StringLength(20)]
        public string Fax { get; set; }
        /// <summary>
        /// כתובת/כתובות המייל
        /// השדה יכול להכיל כתובת אחת או מספר כתובות מופרדות בפסיק
        /// </summary>
        [DataMember, StringLength(100)]
        public virtual string Email { get; set; }

        /// <summary>
        /// סיווג התנועה ברירת מחדל של הלקוח.
        /// ה-API משתמש ב-TrxCodeNumber בכדי לזהות את סיווג התנועה
        /// בעדכון של לקוח: אם הערך שונה מ-NULL אז ה-API יעדכן את הערך בשדה בכספית.
        /// </summary>
        [DataMember]
        public int? TrxCodeNumber { get; set; }
        /// <summary>
        /// השם של סיווג התנועה. משמש רק בשליפה של לקוח מכספית
        /// ולא משמש בעדכון או הוספה של לקוח לכספית
        /// </summary>
        [DataMember, StringLength(50)]
        public string TrxCode { get; set; }
        
        /// <summary>
        /// יתרת פתיחה
        /// בעדכון של לקוח: אם הערך שונה מ-NULL אז ה-API יעדכן את הערך בשדה בכספית.
        /// </summary>
        [DataMember]
        public virtual decimal? OpenBalance { get; set; }
        /// <summary>
        /// יתרה נוכחית מבוססת על חשבוניות מס וקבלות ללא חשבוניות עסקה
        /// בעדכון של לקוח: אם הערך שונה מ-NULL אז ה-API יעדכן את הערך בשדה בכספית.
        /// </summary>
        [DataMember]
        public virtual decimal? Balance { get; set; }
        /// <summary>
        /// יתרה נוכחית מבוססת על חשבוניות מס , קבלות וחשבוניות עסקה
        /// בעדכון של לקוח: אם הערך שונה מ-NULL אז ה-API יעדכן את הערך בשדה בכספית.
        /// </summary>
        [DataMember]
        public virtual decimal? BalanceInvoiceIska { get; set; }

        /// <summary>
        /// המספר (לא ה-ID) של המחירון.
        /// לדוגמה: 2
        /// אם לא נשלח ערך ברירת המחדל היא 1.
        /// </summary>
        [DataMember]
        public virtual int? PriceListNum { get; set; }

        /// <summary>
        /// הערות שונות 1
        /// </summary>
        [DataMember, StringLength(500)]
        public virtual string Comments1 { get; set; }
        /// <summary>
        /// הערות שונות 2
        /// </summary>
        [DataMember, StringLength(500)]
        public virtual string Comments2 { get; set; }
        /// <summary>
        /// הערות שונות 3
        /// </summary>
        [DataMember, StringLength(2000)]
        public virtual string Comments3 { get; set; }
        /// <summary>
        /// הערה לחשבונית - זו הערה שנשתלת אוטומטית בעת יצירת חשבונית ללקוח
        /// לדוגמה: "מספר ספק: 838389"
        /// </summary>
        [DataMember, StringLength(500)]
        public virtual string InvoiceComment { get; set; }

        /// <summary>
        /// פרטי חשבון הבנק
        /// </summary>
        [DataMember, StringLength(20)]
        public virtual string BankAcctNumber { get; set; }
        /// <summary>
        /// סניף הבנק
        /// </summary>
        [DataMember, StringLength(20)]
        public virtual string BankAcctBranch { get; set; }
        /// <summary>
        /// מספר הבנק -- השתמש במספר הבנק לפי רשימת הבנקים של בנק ישראל
        /// </summary>
        [DataMember]
        public virtual int? BankAcctBankId { get; set; }
        /// <summary>
        /// האם לגבות מעמ בעת יצירת מסמכים ללקוח זה
        /// משמש את כספית בעת יצירת חשבונית וכו' ללקוח
        /// </summary>
        [DataMember]
        public virtual bool? ChargeVAT { get; set; }

        /// <summary>
        /// האם הלקוח ישראלי
        /// כספית משתמשת בשדה כשהיא מציגה את פרטי הלקוח בחשבונית.
        /// ללקוח ישראלי לא יוצג השדה מדינה
        /// </summary>
        [DataMember]
        public virtual bool IsIsraeli { get; set; }

        /// <summary>
        /// האם הלקוח "זמני"
        /// לקוח זמני לא מוצג ברשימת הלקוחות בהשלמה אוטומטית בבחירת לקוח בחשבוניות
        /// </summary>
        [DataMember]
        public virtual bool IsOccasional { get; set; }

        /// <summary>
        /// GET, POST:
        /// זיהוי של השותף העיסקי שיזם את הלקוח
        /// PUT:
        /// If null then Caspit will set the Contact's AffiliateId to null
        /// If "" (empty string) then Caspit will NOT update  Contact's AffiliateId
        /// </summary>
        [DataMember, StringLength(50)]
        public string AffiliateId { get; set; }

        [DataMember, StringLength(60)]
        public virtual string CCNumberToken { get; set; }

        [StringLength(9, ErrorMessage = "שדה מספר ת\"ז של בעל הכרטיס מוגבל ל-9 תווים")]
        public virtual string CCHolderTZ { get; set; }

        /// <summary>
        /// תוקף הכרטיס
        /// </summary>
        [DataMember, RegularExpression("^(0[1-9]|1[0-2])\\/?([0-9]{4}|[0-9]{2})$", ErrorMessage = "תוקף הכרטיס לא תקין. הכנס התוקף בפורמט MMYY")]
        [StringLength(4, ErrorMessage = "שדה תוקף הכרטיס מוגבל ל-4 תווים: MMYY")]
        public virtual string CCExpDate { get; set; }

        /// <summary>
        /// CVV2
        /// </summary>
        [DataMember, StringLength(4, ErrorMessage = "שדה CVV מוגבל ל-4 תווים")]
        public virtual string CVV { get; set; }

    }
}