﻿using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel;
using DataAnnotationsExtensions;

namespace CaspitWebTiles.Api.Models
{
    /// <summary>
    /// Represents a Caspit document (e.g. חשבונית מס, חש/קבלה, קבלה, תעודת משלוח)
    /// An invoice will only have DocumentLines, a receipt will only have ReceiptLines, a חש/קבלה will have both
    /// </summary>
    [DataContract(Namespace = "http://www.caspit.biz/2013/03")]
    public class CaspitDocument : CaspitBase, IValidatableObject
    {
        /// <summary>
        /// Represents a Caspit document (e.g. חשבונית מס, חש/קבלה, קבלה, תעודת משלוח)
        /// </summary>
        public CaspitDocument()
        {
            DocumentLines = new List<CaspitDocumentLine>();
            ReceiptLines = new List<CaspitReceiptLine>();
        }

        /// <summary>
        /// The document's Id. You need to supply a unique Id that you will use to fetch or update the document.
        /// </summary>
        [DataMember, StringLength(50)]
        public virtual string DocumentId { get; set; }
        

        /// <summary>
        /// The document's branch number.
        /// </summary>
        [DataMember]
        public virtual int? DocumentNumberBranch { get; set; }

        /// <summary>
        /// See the enumeration in ApiGlobals.TrxTypes 
        /// </summary>
        [DataMember(IsRequired = true)]
        public virtual int? TrxTypeId { get; set; }   //

        /// <summary>
        /// מספר המסמך. לדוגמה: 09/400256
        ///לא בשימוש ביצירה/עדכון מאחר שכספית ממספרת את המסמכים באופן אוטומטי ולא ניתן לשנות את המספר של מסמך קיים. 
        ///כלומר, השדה משמש רק בשליפה של נתונים.
        /// </summary>
        [DataMember, StringLength(50)]
        public string Number { get; set; }

        /// <summary>
        /// תאריך המסמך
        /// </summary>
        [DataMember]
        public DateTime Date { get; set; }

        /// <summary>
        /// זיהוי של השותף העיסקי שיזם את החשבונית
        /// </summary>
        [StringLength(50)]
        public string AffiliateId { get; set; }

        /// <summary>
        /// The document's customer id. 
        /// </summary>
        [DataMember, StringLength(50)]
        public string CustomerId { get; set; }

        /// <summary>
        /// Customer's business name
        /// </summary>
        [DataMember, StringLength(200)]
        public string CustomerBusinessName { get; set; }

        /// <summary>
        /// מספר עוסק מורשה של הלקוח
        /// </summary>
        [DataMember, StringLength(20)]
        public string CustomerOsekMorshe { get; set; }
        /// <summary>
        /// שם איש הקשר -- ללקוח בכספית יכולים להיות 0, 1 או יותר אנשי קשר
        /// </summary>
        [DataMember, StringLength(100)]
        public string CustomerContactName { get; set; }
        /// <summary>
        /// שורת כתובת1: שם הרחוב ומספר הבית
        /// </summary>
        [DataMember, StringLength(100)]
        public string CustomerAddress1 { get; set; }
        /// <summary>
        /// שורת כתובת2: ת.ד., דירה, שכונה וכו'
        /// </summary>
        [DataMember, StringLength(100)]
        public string CustomerAddress2 { get; set; }
        /// <summary>
        /// שם הישוב
        /// </summary>
        [DataMember, StringLength(100)]
        public string CustomerCity { get; set; }
        /// <summary>
        /// המיקוד
        /// </summary>
        [DataMember, StringLength(20)]
        public string CustomerPostalCode { get; set; }
        /// <summary>
        /// כתובת המייל של איש הקשר
        /// השדה יכול להכיל כתובת אחת או מספר כתובות מופרדות בפסיק
        /// </summary>
        [DataMember, StringLength(100)]
        public string CustomerEmail { get; set; }
        /// <summary>
        /// מספר הטלפון הנייד של איש הקשר
        /// </summary>
        [DataMember, StringLength(25)]
        public string MobilePhone { get; set; }

        /// <summary>
        /// פרטים שלא מוצגים ללקוח
        /// </summary>
        [DataMember, StringLength(1000)]
        public virtual string Details { get; set; }

        /// <summary>
        /// הערות שכן מוצגות ללקוח
        /// </summary>
         [DataMember, StringLength(1000)]
        public virtual string Comments { get; set; }

        /// <summary>
        /// סה"כ לפני מע"מ של שורות החשבונית (לאחר הנחת שורה), ולפני הנחות מסמך (=הנחה סתם, המחת עיגול) של הסכום שיש עליו מע"מ
        ///לא בשימוש ביצירה/עדכון. מאוכלס בשליפה
        /// </summary>
        [DataMember]
        public decimal TotalLinesChargeVAT { get; set; }

        /// <summary>
        /// ההנחה בש"ח שיש במסמך.
        /// ההנחה היא מהסכום שיש עליו מעמ.
        /// כלומר לא ניתן לתת הנחה מהסכום שאין עליו מע"מ. 
        /// במקרה כזה תן את ההנחה בשורת המסמך המתאימה.
        /// </summary>
        [DataMember]
        public decimal? Rebate { get; set; }

        /// <summary>
        /// ההנחה באחוזים שיש במסמך.
        /// ההנחה היא מהסכום שיש עליו מעמ.
        /// כלומר לא ניתן לתת הנחה מהסכום שאין עליו מע"מ. 
        /// במקרה כזה תן את ההנחה בשורת המסמך המתאימה.
        /// </summary>
        [DataMember]
        public decimal? RebatePercent { get; set; }

        /// <summary>
        /// אם תשלח false
        /// </summary>
        [DataMember, DefaultValue(false)]
        public virtual bool? RountTotal { get; set; }        
        /// <summary>
        /// הנחת עיגול בש"ח
        /// ההנחה יכולה להיות שלילית בכדי לעגל את סכום המסמך כלפי מעלה.
        ///לא בשימוש ביצירה/עדכון. מאוכלס בשליפה
        /// </summary>
        [DataMember]
        public decimal? RebateRounding { get; set; }

        [DataMember]
        public decimal? TotalBeforeVAT { get; set; }

        /// <summary>
        /// סה"כ לפני מע"מ ואחרי הנחות (=הנחה סתם, המחת עיגול) של הסכום שיש עליו מע"מ
        ///לא בשימוש ביצירה/עדכון. מאוכלס בשליפה
        /// </summary>
        [DataMember]
        public decimal? TotalBeforeVATChargeVAT { get; set; }

        /// <summary>
        /// סה"כ הסכום בש"ח שאין עליו מע"מ
        ///לא בשימוש ביצירה/עדכון. מאוכלס בשליפה
        /// </summary>
        [DataMember, DefaultValue(0.0)]
        public decimal? TotalNoChargeVat { get; set; }

        /// <summary>
        /// המעמ בש"ח שנגבה במסמך
        ///לא בשימוש ביצירה/עדכון. מאוכלס בשליפה
        /// </summary>
        [DataMember]
        public decimal Vat { get; set; }
        /// <summary>
        /// סה"כ הסכום כולל מע"מ שנגבה במסמך
        ///לא בשימוש ביצירה/עדכון. מאוכלס בשליפה
        /// </summary>
        [DataMember]
        public decimal? Total { get; set; }

        /// <summary>
        /// אחוז המעמ בחשבונית (17 מסמל 17%)
        /// אם תשלח NULL אז השדה יאוכלס בערך המתאים לתאריך המסמך.
        /// </summary>
        [DataMember]
        public decimal? VatRate { get; set; }

        #region  *** פרטים של הקבלה ***
        /// <summary>
        /// סכום התשלום
        /// </summary>
        [DataMember]
        public decimal? Payment { get; set; }

        /// <summary>
        /// הניכוי במקור
        /// </summary>
        [DataMember]
        public decimal? TaxDeduction { get; set; }
        /// <summary>
        /// סה"כ שולם. שם לב, זהו הסכום הנומינלי ששולם.
        /// כלומר, אם שולמו 40 דולר הערך בשדה יהיה 40 (ולא השווי בש"ח)
        /// בכדי לקבל את הסכום ששולם בש"ח השתמש ב-TotalPaymentNIS
        /// </summary>
        [DataMember]
        public decimal? TotalPayment
        {
            get
            {
                return Payment + TaxDeduction;
            }
            private set { }
        }
        /// <summary>
        /// המטבע עבור חלק הקבלה
        /// הסמלים הם ₪, $, €. יתכן שהמשתמש הוסיף גם מטבעות משלו
        /// </summary>
        [DataMember, StringLength(5)]
        public string ReceiptCurrencySymbol { get; set; }

        /// <summary>
        /// שער חליפין עבור חלק הקבלה
        /// </summary>
        [DataMember]
        public decimal? ReceiptRate { get; set; }

        /// <summary>
        /// ניכוי במקור בש"ח
        /// מחושב לפי הסכום הנומינלי מוכפל בשער החליפין
        /// </summary>
        [DataMember, DisplayFormat(DataFormatString = "{0:#,##0.00}")]
        public virtual decimal? TaxDeductionNIS
        {
            get
            {
                return TaxDeduction * ReceiptRate;
            }

            private set { }
        }
        /// <summary>
        /// שולם אחרי ניכוי במקור בש"ח
        /// מחושב לפי הסכום הנומינלי מוכפל בשער החליפין
        /// </summary>
        [DataMember, DisplayFormat(DataFormatString = "{0:#,##0.00}")]
        public decimal? PaymentNIS
        {
            get
            {
                return Payment * ReceiptRate;
            }

            private set { }
        }
        /// <summary>
        /// סה"כ שולם בש"ח
        /// מחושב לפי סה"כ התשלום הנומינלי מוכפל בשער החליפין
        /// </summary>
        [DataMember, DisplayFormat(DataFormatString = "{0:#,##0.00}")]
        public virtual decimal? TotalPaymentNIS
        {
            get
            {
                return TotalPayment * ReceiptRate;
            }

            private set { }
        }
        #endregion

        /// <summary>
        /// שורות המסמך
        /// </summary>
        [DataMember]
        public virtual List<CaspitDocumentLine> DocumentLines { get; set; }

        [DataMember]
        public virtual ICollection<CaspitReceiptLine> ReceiptLines { get; set; }
        /// <summary>
        /// שמו של סיווג תנועה של המסמך. לדוגמה: "מכירות"
        /// </summary>
        [DataMember, StringLength(50)]
        public string TrxCode { get; set; }
        /// <summary>
        /// מספרו של סיווג תנועה של המסמך. לדוגמה: 3
        /// </summary>
        [DataMember]
        public int? TrxCodeNumber { get; set; }

        /// <summary>
        /// See Globals.DocStatus
        /// </summary>
        [DataMember]
        ///לא בשימוש ביצירה/עדכון. מאוכלס בשליפה
        public int? Status { get; set; }

        /// <summary>
        /// תאריך תשלום צפוי
        /// </summary>
        [DataMember]
        public virtual DateTime? DueDate { get; set; }

        /// <summary>
        /// This field is used to identify changes made to the Document by other user
        /// When you update a socumend send the RowVersion value that you received when you fetched the document
        /// </summary>
        [DataMember, Timestamp]
        public byte[] RowVersion { get; set; }

        /// <summary>
        /// A link to the document's PDF
        /// </summary>
        [DataMember]
        public string LinkToPdf { get; set; }

        /// <summary>
        /// A url which displayes the document's PDF
        /// The field has a value only if the document was sent in the email
        /// </summary>
        [DataMember]
        public string ViewUrl { get; set; }

        /// <summary>
        /// A URL to a payment page for an Invoice, Invoice Iska, Invoice Kabala
        /// The field has value only if the business has a valid license to charge credit cards via Caspit
        /// </summary>
        [DataMember]
        public string PaymentUrl { get; set; }

        public IEnumerable<ValidationResult> Validate(ValidationContext validationContext)
        {
            return new List<ValidationResult>();
        }

    }

    [DataContract(Namespace = "http://www.caspit.biz/2013/03")]
    public class CaspitDocumentLine1
    {
        /// <summary>
        /// מספר השורה במסמך. 
        /// חובה לשלוח את מספר השורה משום שהוא משמש לזיהוי השורה הרלוונטית במסמך שכבר קיים בכספית
        /// </summary>
        [DataMember(IsRequired = true)]
        public int Number { get; set; }
    }

    [DataContract(Namespace = "http://www.caspit.biz/2013/03")]
    public class CaspitDocumentLine
    {
        /// <summary>
        /// מספר השורה במסמך. 
        /// חובה לשלוח את מספר השורה משום שהוא משמש לזיהוי השורה הרלוונטית במסמך שכבר קיים בכספית
        /// </summary>
        [DataMember(IsRequired=true)]
        public int Number { get; set; }

        /// <summary>
        /// תאריך תשלום צפוי
        /// </summary>
        [DataType(DataType.DateTime, ErrorMessage="הערך חייב להיות תאריך חוקי")]
        public virtual DateTime Date { get; set; }

        /// <summary>
        /// המפתח של הפריט. משמש בכספית+מלאי לניהול המלאי ולדוחות מכירה של פריטים.
        /// בכספית+ מקשר בין שורות המסמך לטבלת פריטים
        /// בגרסאות אחרות אפשר לשלוח NULL.
        /// </summary>
        [DataMember, StringLength(50)]
        public virtual string ProductId { get; set; }

        /// <summary>
        /// שם הפריט
        /// </summary>
        [DataMember, StringLength(4000)]
        public string ProductName { get; set; }
        /// <summary>
        /// פרטי הפריט או השירות שנמכר
        /// </summary>
        [DataMember, StringLength(4000)]
        public string Details { get; set; }
        /// <summary>
        /// מחיר יחידה
        /// </summary>
        [DataMember(IsRequired = true)]
        public decimal UnitPrice { get; set; }
        /// <summary>
        /// הכמות שנמכרה
        /// </summary>
        [DataMember(IsRequired = true)]
        public virtual decimal Qty { get; set; }
        
        /// <summary>
        /// סמל המטבע - הסמלים האפשריים הם הסמלים שטבלת המטבעות של כספית.
        /// ראה במסך הראשי, מסך מטבעות. לדוגמה: ₪, $, €, A$
        /// </summary>
        [DataMember, StringLength(4), DefaultValue("₪")]
        public string CurrencySymbol { get; set; }
        /// <summary>
        /// שער החליפין. עבור ש"ח השער הוא 1.0
        /// </summary>
        [DataMember, DefaultValue(1.000)]
        public decimal Rate { get; set; }

        /// <summary>
        /// הנחת שורה בש"ח (או בדולרים וכו'). לא באחוזים.
        /// </summary>
        [DataMember, DefaultValue(0.000)]
        public decimal Rebate { get; set; }

        /// <summary>
        /// מחיר כולל של השורה לאחר הנחה ולפני מע"מ
        /// </summary>
        [DataMember]
        public decimal ExtendedPrice
        {
            get
            {
                return UnitPrice * Qty * Rate - Rebate;
            }
            private set {}
        }

        /// <summary>
        /// האם לגבות מע"מ על השורה
        /// </summary>
        [DataMember, DefaultValue(true)]
        public virtual bool ChargeVAT { get; set; }
        /// <summary>
        /// אחוז המעמ בשורה (17 מסמל 17%)
        ///לא בשימוש ביצירה/עדכון (הערך נלקח מהחשבונית). מאוכלס בשליפה
        /// </summary>
        [DataMember]
        public virtual decimal? VATRate { get; set; }

        /// <summary>
        /// המעמ (בש"ח) שיש לגבות על השורה
        ///לא בשימוש ביצירה/עדכון. מאוכלס בשליפה
        /// </summary>
        [DataMember]
        public virtual decimal? VAT { get; set; }
    }

    /// <summary>
    /// Represents a line in a Caspit receipt (קבלה)
    /// Note: Since we use the same fields to represent both check payments and a credit card payments some of the fields have
    /// multiple semantic meanings.
    /// </summary>
    [DataContract(Namespace = "http://www.caspit.biz/2013/03")]
    public class CaspitReceiptLine: IValidatableObject
    {
        /// <summary>
        /// מספר השורה במסמך. 
        /// חובה לשלוח את מספר השורה משום שהוא משמש לזיהוי השורה הרלוונטית במסמך שכבר קיים בכספית
        /// </summary>
        [Required, DataMember(IsRequired=true)]
        public int Number { get; set; }

        /// <summary>
        /// שיק, מזומן וכו'
        /// See the enumeration ApiGlobals.PaymentTypes in the SDK
        /// </summary>
        [Required, DataMember]
        public int PaymentTypeId { get; set; }

        /// <summary>
        /// מספר החשבון או כרטיס האשראי
        /// </summary>
        [DataMember, StringLength(50)]
        public string AccountNumber { get; set; }

        /// <summary>
        /// מספר הסניף או תוקף הכרטיס
        /// </summary>
        [DataMember, StringLength(50)]
        public virtual string BranchOrValidTo { get; set; }

        /// <summary>
        /// הבנק (ה-id נלקח ממרשימת הבנקים של בנק ישראל(
        /// </summary>
        [DataMember]
        public int? BankId { get; set; }

        /// <summary>
        /// מספר השיק או השובר
        /// </summary>
        [DataMember, StringLength(50)]
        public string CheckNumber { get; set; }

        /// <summary>
        /// תאריך השיק או תאריך הזיכוי של כרטיס האשראי
        /// </summary>
        [DataMember]
        public DateTime? PaymentDate { get; set; }

        /// <summary>
        /// סכום התשלום הנומינלי (כלומר, התשלום במטבע של הקבלה)
        /// אם לדוגמה התשלום הוא 400 דולר, הערך כאן יהיה 400 והמטבע של הקבלה יהיה $.
        /// סכום התשלום בש"ח מוצג בשדה TotalPaymentNIS ברמת הקבלה.
        /// </summary>
        [DataMember]
        public decimal? Payment { get; set; }

        /// <summary>
        /// האם ניתן לעדכן את השורה
        /// </summary>
        [DataMember]
        public bool? CanBeChanged { get; set; }

        public IEnumerable<ValidationResult> Validate(ValidationContext validationContext)
        {
            if (this.Payment < 0)
            {
                yield return new ValidationResult(string.Format("הסכום בשורה {0} הוא {1} קטן מ-0. כל הסכומים בשורות קבלה חייבים להיות גדולים/שווים ל-0, גם בתעודת החזר כספים.",
                    this.Number, this.Payment),
                    new[] { "Payment" });
            }
        }
    }

}